@@ -14,17 +14,12 @@
 {-# LANGUAGE ViewPatterns #-}
 #endif
 
-#ifndef MIN_VERSION_base
-#define MIN_VERSION_base(x,y,z) 1
-#endif
-
-#ifndef MIN_VERSION_exceptions
-#define MIN_VERSION_exceptions 1
-#endif
+#include "lens-common.h"
 
 #if !(MIN_VERSION_exceptions(0,4,0))
 #define MonadThrow MonadCatch
 #endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Exception.Lens
@@ -73,9 +68,7 @@ module Control.Exception.Lens
   -- ** Arithmetic Exceptions
   , AsArithException(..)
   , _Overflow, _Underflow, _LossOfPrecision, _DivideByZero, _Denormal
-#if MIN_VERSION_base(4,6,0)
   , _RatioZeroDenominator
-#endif
 #if __GLASGOW_HASKELL__ >= 710
   , pattern ArithException_
   , pattern Overflow_
@@ -218,12 +211,10 @@ import Prelude
 #endif
   )
 
-#ifdef HLINT
-{-# ANN module "HLint: ignore Use Control.Exception.catch" #-}
-#endif
-
 -- $setup
 -- >>> :set -XNoOverloadedStrings
+-- >>> import Control.Lens
+-- >>> import Control.Applicative
 -- >>> :m + Control.Exception Control.Monad Data.List Prelude
 
 ------------------------------------------------------------------------------
@@ -268,7 +259,7 @@ catching l = catchJust (preview l)
 {-# INLINE catching #-}
 
 -- | Catch exceptions that match a given 'Prism' (or any 'Getter'), discarding
--- the information about the match. This is particuarly useful when you have
+-- the information about the match. This is particularly useful when you have
 -- a @'Prism'' e ()@ where the result of the 'Prism' or 'Fold' isn't
 -- particularly valuable, just the fact that it matches.
 --
@@ -637,10 +628,7 @@ pattern Denormal_ <- (has _Denormal -> True) where
   Denormal_ = review _Denormal ()
 #endif
 
-#if MIN_VERSION_base(4,6,0)
--- | Added in @base@ 4.6 in response to this libraries discussion:
---
--- <http://haskell.1045720.n5.nabble.com/Data-Ratio-and-exceptions-td5711246.html>
+-- |
 --
 -- @
 -- '_RatioZeroDenominator' ≡ '_ArithException' '.' '_RatioZeroDenominator'
@@ -661,8 +649,6 @@ pattern RatioZeroDenominator_ <- (has _RatioZeroDenominator -> True) where
   RatioZeroDenominator_ = review _RatioZeroDenominator ()
 #endif
 
-#endif
-
 ----------------------------------------------------------------------------
 -- ArrayException
 ----------------------------------------------------------------------------
