@@ -1,13 +1,7 @@
 {-# LANGUAGE CPP #-}
-
-#if __GLASGOW_HASKELL__ >= 708
-#define USE_COERCE
 {-# LANGUAGE Trustworthy #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-#else
-{-# LANGUAGE Unsafe #-}
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Copyright   :  (C) 2016 Edward Kmett and Eric Mertens
@@ -16,9 +10,8 @@
 -- Stability   :  experimental
 -- Portability :  non-portable
 --
--- This module provides a shim around 'coerce' that defaults to 'unsafeCoerce'
--- on GHC < 7.8. It also exposes a type-restricted version of '(#.)' that
--- works around a bizarre GHC 7.10–specific bug.
+-- This module provides a shim around 'coerce'.
+-- We need to work around a GHC 7.8-specific 'Coercible' solver bug(s).
 -----------------------------------------------------------------------------
 module Control.Lens.Internal.Coerce
   ( coerce
@@ -27,30 +20,16 @@ module Control.Lens.Internal.Coerce
   ) where
 
 import Data.Profunctor.Unsafe
-
-#ifdef USE_COERCE
-
 import Data.Coerce
 
 coerce' :: forall a b. Coercible a b => b -> a
+#if __GLASGOW_HASKELL__ <710
 coerce' = coerce (id :: a -> a)
+#else
+coerce' = coerce
+#endif
 {-# INLINE coerce' #-}
 
 (#..) :: (Profunctor p, Coercible c b) => (b -> c) -> p a b -> p a c
 (#..) = (#.)
 {-# INLINE (#..) #-}
-
-#else
-
-import Unsafe.Coerce
-
-coerce, coerce' :: a -> b
-coerce  = unsafeCoerce
-coerce' = unsafeCoerce
-{-# INLINE coerce #-}
-{-# INLINE coerce' #-}
-
-(#..) :: Profunctor p => (b -> c) -> p a b -> p a c
-(#..) = (#.)
-{-# INLINE (#..) #-}
-#endif
