@@ -23,8 +23,6 @@ module Control.Lens.Internal.Fold
   , Traversed(..)
   , TraversedF(..)
   , Sequenced(..)
-  , Max(..), getMax
-  , Min(..), getMin
   , Leftmost(..), getLeftmost
   , Rightmost(..), getRightmost
   , ReifiedMonoid(..)
@@ -32,21 +30,16 @@ module Control.Lens.Internal.Fold
   , NonEmptyDList(..)
   ) where
 
-import Control.Applicative
+import Prelude ()
+
 import Control.Lens.Internal.Getter
+import Control.Lens.Internal.Prelude
 import Data.Functor.Bind
-import Data.Functor.Contravariant
-import Data.Maybe
-import Data.Semigroup hiding (Min, getMin, Max, getMax)
+import Data.Maybe (fromMaybe)
 import Data.Reflection
-import Prelude
 
 import qualified Data.List.NonEmpty as NonEmpty
 
-#ifdef HLINT
-{-# ANN module "HLint: ignore Avoid lambda" #-}
-#endif
-
 ------------------------------------------------------------------------------
 -- Folding
 ------------------------------------------------------------------------------
@@ -124,60 +117,6 @@ instance Monad m => Monoid (Sequenced a m) where
   Sequenced ma `mappend` Sequenced mb = Sequenced (ma >> mb)
   {-# INLINE mappend #-}
 
-------------------------------------------------------------------------------
--- Min
-------------------------------------------------------------------------------
-
--- | Used for 'Control.Lens.Fold.minimumOf'.
-data Min a = NoMin | Min a
-
-instance Ord a => Semigroup (Min a) where
-  NoMin <> m     = m
-  m <> NoMin     = m
-  Min a <> Min b = Min (min a b)
-  {-# INLINE (<>) #-}
-
-instance Ord a => Monoid (Min a) where
-  mempty = NoMin
-  {-# INLINE mempty #-}
-  mappend NoMin m = m
-  mappend m NoMin = m
-  mappend (Min a) (Min b) = Min (min a b)
-  {-# INLINE mappend #-}
-
--- | Obtain the minimum.
-getMin :: Min a -> Maybe a
-getMin NoMin   = Nothing
-getMin (Min a) = Just a
-{-# INLINE getMin #-}
-
-------------------------------------------------------------------------------
--- Max
-------------------------------------------------------------------------------
-
--- | Used for 'Control.Lens.Fold.maximumOf'.
-data Max a = NoMax | Max a
-
-instance Ord a => Semigroup (Max a) where
-  NoMax <> m = m
-  m <> NoMax = m
-  Max a <> Max b = Max (max a b)
-  {-# INLINE (<>) #-}
-
-instance Ord a => Monoid (Max a) where
-  mempty = NoMax
-  {-# INLINE mempty #-}
-  mappend NoMax m = m
-  mappend m NoMax = m
-  mappend (Max a) (Max b) = Max (max a b)
-  {-# INLINE mappend #-}
-
--- | Obtain the maximum.
-getMax :: Max a -> Maybe a
-getMax NoMax   = Nothing
-getMax (Max a) = Just a
-{-# INLINE getMax #-}
-
 ------------------------------------------------------------------------------
 -- NonEmptyDList
 ------------------------------------------------------------------------------
