@@ -18,10 +18,6 @@ module Control.Lens.Internal.List
 import Data.IntSet (IntSet)
 import qualified Data.IntSet as IntSet
 
-#ifdef HLINT
-{-# ANN module "HLint: ignore Redundant bracket" #-}
-#endif
-
 -- | Return the the subset of given ordinals within a given bound
 -- and in order of the first occurrence seen.
 --
