@@ -4,10 +4,6 @@
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE Trustworthy #-}
 
-#if __GLASGOW_HASKELL__ >= 800
-{-# OPTIONS_GHC -Wno-trustworthy-safe #-}
-#endif
-
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-warnings-deprecations #-}
 -----------------------------------------------------------------------------
 -- |
@@ -34,15 +30,12 @@ module Control.Lens.Internal.Zoom
   , EffectRWS(..)
   ) where
 
-import Control.Applicative
-import Control.Category
-import Control.Comonad
-import Control.Monad.Reader as Reader
+import Prelude ()
+
+import Control.Lens.Internal.Prelude
+import Control.Monad
 import Control.Monad.Trans.Free
 import Data.Functor.Bind
-import Data.Functor.Contravariant
-import Data.Semigroup
-import Prelude hiding ((.),id)
 
 ------------------------------------------------------------------------------
 -- Focusing
