@@ -44,6 +44,7 @@ import Data.Semigroup
 
 -- $setup
 -- >>> import Control.Lens
+-- >>> import Control.Applicative
 
 ------------------------------------------------------------------------------
 -- Lens
@@ -256,7 +257,7 @@ instance Profunctor (ReifiedIndexedGetter i) where
   {-# INLINE dimap #-}
 
 instance Sieve (ReifiedIndexedGetter i) ((,) i) where
-  sieve = iview . runIndexedGetter
+  sieve (IndexedGetter l) = iview l
   {-# INLINE sieve #-}
 
 instance Representable (ReifiedIndexedGetter i) where
@@ -307,7 +308,7 @@ instance Profunctor ReifiedFold where
   {-# INLINE lmap #-}
 
 instance Sieve ReifiedFold [] where
-  sieve = toListOf . runFold
+  sieve (Fold l) = toListOf l
 
 instance Representable ReifiedFold where
   type Rep ReifiedFold = []
