@@ -8,13 +8,8 @@
 # endif
 #endif
 
-#ifndef MIN_VERSION_base
-#define MIN_VERSION_base(x,y,z) 1
-#endif
+#include "lens-common.h"
 
-#ifndef MIN_VERSION_template_haskell
-#define MIN_VERSION_template_haskell(x,y,z) (defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 706)
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.TH
@@ -84,12 +79,8 @@ module Control.Lens.TH
   , abbreviatedNamer
   ) where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative
-#endif
-#if !(MIN_VERSION_template_haskell(2,7,0))
-import Control.Monad (ap)
-#endif
+import Prelude ()
+
 import Control.Monad.Trans.Class
 import Control.Monad.Trans.State
 import Control.Monad.Trans.Writer
@@ -99,6 +90,7 @@ import Control.Lens.Lens
 import Control.Lens.Setter
 import Control.Lens.Tuple
 import Control.Lens.Traversal
+import Control.Lens.Internal.Prelude as Prelude
 import Control.Lens.Internal.TH
 import Control.Lens.Internal.FieldTH
 import Control.Lens.Internal.PrismTH
@@ -106,25 +98,22 @@ import Control.Lens.Wrapped () -- haddocks
 import Control.Lens.Type () -- haddocks
 import Data.Char (toLower, toUpper, isUpper)
 import Data.Foldable hiding (concat, any)
-import Data.List as List
+import qualified Data.List as List
 import qualified Data.Map as Map
 import Data.Map (Map)
 import Data.Maybe (maybeToList)
-import Data.Monoid
 import qualified Data.Set as Set
 import Data.Set (Set)
 import Data.Set.Lens
 import Data.Traversable hiding (mapM)
-import Language.Haskell.TH
 import Language.Haskell.TH.Datatype
+import Language.Haskell.TH.Datatype.TyVarBndr as D
 import Language.Haskell.TH.Lens
-import Language.Haskell.TH.Syntax hiding (lift)
-
-#ifdef HLINT
-{-# ANN module "HLint: ignore Eta reduce" #-}
-{-# ANN module "HLint: ignore Use fewer imports" #-}
-{-# ANN module "HLint: ignore Use foldl" #-}
+import Language.Haskell.TH.Lib
+#if MIN_VERSION_template_haskell(2,15,0)
+import Language.Haskell.TH.Ppr (pprint)
 #endif
+import Language.Haskell.TH.Syntax hiding (lift)
 
 -- | Generate "simple" optics even when type-changing optics are possible.
 -- (e.g. 'Lens'' instead of 'Lens')
@@ -525,7 +514,7 @@ makeDataDecl dec = case deNewtype dec of
                     -> Just DataDecl
     { dataContext = ctx
     , tyConName = Nothing
-    , dataParameters = map PlainTV vars
+    , dataParameters = map D.plainTV vars
     , fullType = \tys -> apps (ConT familyName) $
         substType (Map.fromList $ zip vars tys) args
     , constructors = cons
@@ -553,7 +542,7 @@ data DataDecl = DataDecl
   { dataContext :: Cxt -- ^ Datatype context.
   , tyConName :: Maybe Name
     -- ^ Type constructor name, or Nothing for a data family instance.
-  , dataParameters :: [TyVarBndr] -- ^ List of type parameters
+  , dataParameters :: [TyVarBndrUnit] -- ^ List of type parameters
   , fullType :: [Type] -> Type
     -- ^ Create a concrete record type given a substitution to
     -- 'detaParameters'.
@@ -639,13 +628,6 @@ makeWrappedInstance dataDecl con fieldType = do
   --   _Wrapped' = iso (\(Con x) -> x) Con
   instanceD (cxt []) klass [unwrappedATF, isoMethod]
 
-#if !(MIN_VERSION_template_haskell(2,7,0))
--- | The orphan instance for old versions is bad, but programming without 'Applicative' is worse.
-instance Applicative Q where
-  pure = return
-  (<*>) = ap
-#endif
-
 overHead :: (a -> a) -> [a] -> [a]
 overHead _ []     = []
 overHead f (x:xs) = f x : xs
@@ -682,7 +664,7 @@ camelCaseFields = defaultFieldRules
 camelCaseNamer :: FieldNamer
 camelCaseNamer tyName fields field = maybeToList $ do
 
-  fieldPart <- stripPrefix expectedPrefix (nameBase field)
+  fieldPart <- List.stripPrefix expectedPrefix (nameBase field)
   method    <- computeMethod fieldPart
   let cls = "Has" ++ fieldPart
   return (MethodName (mkName cls) (mkName method))
@@ -690,7 +672,7 @@ camelCaseNamer tyName fields field = maybeToList $ do
   where
   expectedPrefix = optUnderscore ++ overHead toLower (nameBase tyName)
 
-  optUnderscore  = ['_' | any (isPrefixOf "_" . nameBase) fields ]
+  optUnderscore  = ['_' | any (List.isPrefixOf "_" . nameBase) fields ]
 
   computeMethod (x:xs) | isUpper x = Just (toLower x : xs)
   computeMethod _                  = Nothing
@@ -709,7 +691,7 @@ classUnderscoreNoPrefixFields =
 -- | A 'FieldNamer' for 'classUnderscoreNoPrefixFields'.
 classUnderscoreNoPrefixNamer :: FieldNamer
 classUnderscoreNoPrefixNamer _ _ field = maybeToList $ do
-  fieldUnprefixed <- stripPrefix "_" (nameBase field)
+  fieldUnprefixed <- List.stripPrefix "_" (nameBase field)
   let className  = "Has" ++ overHead toUpper fieldUnprefixed
       methodName = fieldUnprefixed
   return (MethodName (mkName className) (mkName methodName))
@@ -735,11 +717,11 @@ abbreviatedNamer _ fields field = maybeToList $ do
   return (MethodName (mkName cls) (mkName method))
 
   where
-  stripMaxLc f = do x <- stripPrefix optUnderscore f
+  stripMaxLc f = do x <- List.stripPrefix optUnderscore f
                     case break isUpper x of
                       (p,s) | List.null p || List.null s -> Nothing
                             | otherwise                  -> Just s
-  optUnderscore  = ['_' | any (isPrefixOf "_" . nameBase) fields ]
+  optUnderscore  = ['_' | any (List.isPrefixOf "_" . nameBase) fields ]
 
   computeMethod (x:xs) | isUpper x = Just (toLower x : xs)
   computeMethod _                  = Nothing
@@ -863,7 +845,7 @@ emit decs = tell $ Endo (decs++)
 -- | Traverse each data, newtype, data instance or newtype instance
 -- declaration.
 traverseDataAndNewtype :: (Applicative f) => (Dec -> f Dec) -> [Dec] -> f [Dec]
-traverseDataAndNewtype f decs = traverse go decs
+traverseDataAndNewtype f = traverse go
   where
     go dec = case dec of
       DataD{} -> f dec
