@@ -8,9 +8,8 @@
 {-# LANGUAGE Trustworthy #-}
 {-# LANGUAGE ConstraintKinds #-}
 
-#ifndef MIN_VERSION_containers
-#define MIN_VERSION_containers(x,y,z) 1
-#endif
+#include "lens-common.h"
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Traversal
@@ -129,63 +128,50 @@ module Control.Lens.Traversal
   , confusing
   ) where
 
-import Control.Applicative as Applicative
+import Prelude ()
+
 import Control.Applicative.Backwards
-import Control.Category
+import qualified Control.Category as C
 import Control.Comonad
 import Control.Lens.Fold
 import Control.Lens.Getter (Getting, IndexedGetting, getting)
 import Control.Lens.Internal.Bazaar
 import Control.Lens.Internal.Context
-import Control.Lens.Internal.Indexed
 import Control.Lens.Internal.Fold
+import Control.Lens.Internal.Indexed
+import Control.Lens.Internal.Prelude
 import Control.Lens.Lens
 import Control.Lens.Setter (ASetter, AnIndexedSetter, isets, sets)
 import Control.Lens.Type
-import Control.Monad
 import Control.Monad.Trans.State.Lazy
 import Data.Bitraversable
 import Data.CallStack
 import Data.Functor.Apply
-import Data.Functor.Compose
 import Data.Functor.Day.Curried
 import Data.Functor.Yoneda
 import Data.Int
-import Data.IntMap as IntMap
-import Data.List.NonEmpty (NonEmpty (..))
+import qualified Data.IntMap as IntMap
 import qualified Data.Map as Map
 import Data.Map (Map)
+import Data.Monoid (Any (..))
 import Data.Sequence (Seq, mapWithIndex)
 import Data.Vector as Vector (Vector, imap)
-import Data.Monoid (Any (..), Endo (..))
-import Data.Profunctor
-import Data.Profunctor.Rep
-import Data.Profunctor.Sieve
-import Data.Profunctor.Unsafe
+import Data.Profunctor.Rep (Representable (..))
 import Data.Reflection
 import Data.Semigroup.Traversable
 import Data.Semigroup.Bitraversable
-import Data.Traversable
 import Data.Tuple (swap)
 import GHC.Magic (inline)
-import Prelude hiding ((.),id)
-
-#if !(MIN_VERSION_base(4,8,0))
-import Data.Foldable (Foldable)
-import Data.Monoid (Monoid (..))
-#endif
-
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Semigroup (Semigroup (..))
-#endif
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings -XFlexibleContexts
 -- >>> import Data.Char (toUpper)
 -- >>> import Control.Lens
+-- >>> import Control.Lens.Internal.Context
 -- >>> import Control.DeepSeq (NFData (..), force)
 -- >>> import Control.Exception (evaluate,try,ErrorCall(..))
 -- >>> import Data.Maybe (fromMaybe)
+-- >>> import Data.List.NonEmpty (NonEmpty (..))
 -- >>> import Debug.SimpleReflect.Vars
 -- >>> import Data.Void
 -- >>> import Data.List (sort)
@@ -645,7 +631,7 @@ singular l = conjoined
     (w:ws) -> unsafeOuts b . (:ws) <$> afb w
     []     -> unsafeOuts b . return <$> afb (error "singular: empty traversal"))
   (\pafb s -> let b = l sell s in case pins b of
-    (w:ws) -> unsafeOuts b . (:Prelude.map extract ws) <$> cosieve pafb w
+    (w:ws) -> unsafeOuts b . (:map extract ws) <$> cosieve pafb w
     []     -> unsafeOuts b . return                    <$> cosieve pafb (error "singular: empty traversal"))
 {-# INLINE singular #-}
 
@@ -693,11 +679,11 @@ pins :: (Bizarre p w, Corepresentable p) => w a b t -> [Corep p a]
 pins = getConst #. bazaar (cotabulate $ \ra -> Const [ra])
 {-# INLINE pins #-}
 
-parr :: (Profunctor p, Category p) => (a -> b) -> p a b
-parr f = lmap f id
+parr :: (Profunctor p, C.Category p) => (a -> b) -> p a b
+parr f = lmap f C.id
 {-# INLINE parr #-}
 
-outs :: (Bizarre p w, Category p) => w a a t -> [a] -> t
+outs :: (Bizarre p w, C.Category p) => w a a t -> [a] -> t
 outs = evalState `rmap` bazaar (parr (state . unconsWithDefault))
 {-# INLINE outs #-}
 
@@ -750,7 +736,7 @@ holeInOne x = Holes $ \xt ->
 {-# INLINABLE holeInOne #-}
 
 -- | The non-empty version of 'holesOf'.
--- This extract a non-empty list of immediate children accroding to a given
+-- This extract a non-empty list of immediate children according to a given
 -- 'Traversal1' as editable contexts.
 --
 -- >>> let head1 f s = runPretext (NonEmpty.head $ holes1Of traversed1 s) f
@@ -773,11 +759,11 @@ holes1Of f xs = flip getNonEmptyDList [] . fst $
   runHoles (runBazaar1 (f sell xs) (cotabulate holeInOne1)) id
 {-# INLINE holes1Of #-}
 
-holeInOne1 :: forall p a t. (Corepresentable p, Category p)
+holeInOne1 :: forall p a t. (Corepresentable p, C.Category p)
           => Corep p a -> Holes t (NonEmptyDList (Pretext p a a t)) a
 holeInOne1 x = Holes $ \xt ->
     ( NonEmptyDList (fmap xt (cosieve sell x) :|)
-    , cosieve (id :: p a a) x)
+    , cosieve (C.id :: p a a) x)
 
 -- We are very careful to share as much structure as possible among
 -- the results (in the common case where the traversal allows for such).
@@ -1175,13 +1161,9 @@ class Ord k => TraverseMin k m | m -> k where
   -- | 'IndexedTraversal' of the element with the smallest index.
   traverseMin :: IndexedTraversal' k (m v) v
 
-instance TraverseMin Int IntMap where
+instance TraverseMin Int IntMap.IntMap where
   traverseMin f m = case IntMap.minViewWithKey m of
-#if MIN_VERSION_containers(0,5,0)
     Just ((k,a), _) -> indexed f k a <&> \v -> IntMap.updateMin (const (Just v)) m
-#else
-    Just ((k,a), _) -> indexed f k a <&> \v -> IntMap.updateMin (const v) m
-#endif
     Nothing     -> pure m
   {-# INLINE traverseMin #-}
 
@@ -1196,13 +1178,9 @@ class Ord k => TraverseMax k m | m -> k where
   -- | 'IndexedTraversal' of the element at the largest index.
   traverseMax :: IndexedTraversal' k (m v) v
 
-instance TraverseMax Int IntMap where
+instance TraverseMax Int IntMap.IntMap where
   traverseMax f m = case IntMap.maxViewWithKey m of
-#if MIN_VERSION_containers(0,5,0)
     Just ((k,a), _) -> indexed f k a <&> \v -> IntMap.updateMax (const (Just v)) m
-#else
-    Just ((k,a), _) -> indexed f k a <&> \v -> IntMap.updateMax (const v) m
-#endif
     Nothing     -> pure m
   {-# INLINE traverseMax #-}
 
@@ -1244,7 +1222,7 @@ elementOf l p = elementsOf l (p ==)
 -- 'element' ≡ 'elementOf' 'traverse'
 -- @
 element :: Traversable t => Int -> IndexedTraversal' Int (t a) a
-element = elementOf traverse
+element i = elementOf traverse i
 {-# INLINE element #-}
 
 -- | Traverse (or fold) selected elements of a 'Traversal' (or 'Fold') where their ordinal positions match a predicate.
@@ -1266,7 +1244,7 @@ elementsOf l p iafb s = snd $ runIndexing (l (\a -> Indexing (\i -> i `seq` (i +
 -- 'elements' ≡ 'elementsOf' 'traverse'
 -- @
 elements :: Traversable t => (Int -> Bool) -> IndexedTraversal' Int (t a) a
-elements = elementsOf traverse
+elements i = elementsOf traverse i
 {-# INLINE elements #-}
 
 -- | Try to map a function over this 'Traversal', failing if the 'Traversal' has no targets.
@@ -1286,7 +1264,7 @@ elements = elementsOf traverse
 failover :: Alternative m => LensLike ((,) Any) s t a b -> (a -> b) -> s -> m t
 failover l afb s = case l ((,) (Any True) . afb) s of
   (Any True, t)  -> pure t
-  (Any False, _) -> Applicative.empty
+  (Any False, _) -> empty
 {-# INLINE failover #-}
 
 -- | Try to map a function which uses the index over this 'IndexedTraversal', failing if the 'IndexedTraversal' has no targets.
@@ -1297,7 +1275,7 @@ failover l afb s = case l ((,) (Any True) . afb) s of
 ifailover :: Alternative m => Over (Indexed i) ((,) Any) s t a b -> (i -> a -> b) -> s -> m t
 ifailover l iafb s = case l ((,) (Any True) `rmap` Indexed iafb) s of
   (Any True, t) -> pure t
-  (Any False, _) -> Applicative.empty
+  (Any False, _) -> empty
 {-# INLINE ifailover #-}
 
 -- | Try the first 'Traversal' (or 'Fold'), falling back on the second 'Traversal' (or 'Fold') if it returns no entries.
