@@ -3,18 +3,11 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
-
-#ifndef MIN_VERSION_mtl
-#define MIN_VERSION_mtl(x,y,z) 1
-#endif
-
-#if __GLASGOW_HASKELL__ < 708
 {-# LANGUAGE Trustworthy #-}
-#endif
-
 {-# LANGUAGE RankNTypes #-}
 
+{-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
+
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Unsound
@@ -37,9 +30,13 @@ module Control.Lens.Unsound
   , adjoin
   ) where
 
-import Control.Applicative
 import Control.Lens
-import Prelude
+import Control.Lens.Internal.Prelude
+import Prelude ()
+
+-- $setup
+-- >>> :set -XNoOverloadedStrings
+-- >>> import Control.Lens
 
 -- | A lens product. There is no law-abiding way to do this in general.
 -- Result is only a valid 'Lens' if the input lenses project disjoint parts of
@@ -84,9 +81,9 @@ lensProduct l1 l2 f s =
 prismSum :: APrism s t a b
          -> APrism s t c d
          -> Prism s t (Either a c) (Either b d)
-prismSum k =
-    withPrism k                  $ \bt seta k' ->
-    withPrism k'                 $ \dt setb    ->
+prismSum k k' =
+    withPrism k                  $ \bt seta ->
+    withPrism k'                 $ \dt setb ->
     prism (either bt dt) $ \s ->
     f (Left <$> seta s) (Right <$> setb s)
   where
