@@ -1,9 +1,7 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE Rank2Types #-}
-#ifndef HLINT
 {-# LANGUAGE UnboxedTuples #-}
-#endif
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE FlexibleContexts #-}
@@ -15,6 +13,12 @@
 {-# LANGUAGE Trustworthy #-}
 #endif
 {-# OPTIONS_GHC -fno-full-laziness #-}
+#if __GLASGOW_HASKELL__ >= 810
+-- Use -fbyte-code explicitly to ensure that -fobject-code isn't automatically
+-- implied on GHCi 8.10+ by the use of UnboxedTuples, as this breaks the
+-- doctests. See #874 for more details.
+{-# OPTIONS_GHC -fbyte-code #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Data.Lens
@@ -68,16 +72,9 @@ import           GHC.Exts (realWorld#)
 import           Prelude
 
 import qualified Data.Proxy as X (Proxy (..))
-import qualified Control.Lens.Internal.Typeable as X
+import qualified Data.Typeable as X (typeRep, eqT)
 import qualified Data.Type.Equality as X
 
-#ifdef HLINT
-{-# ANN module "HLint: ignore Eta reduce" #-}
-{-# ANN module "HLint: ignore Use foldl" #-}
-{-# ANN module "HLint: ignore Reduce duplication" #-}
-{-# ANN module "HLint: ignore Unused LANGUAGE pragma" #-}
-#endif
-
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Control.Lens
@@ -326,13 +323,11 @@ fixEq f = go where
        where x' = f x
 {-# INLINE fixEq #-}
 
-#ifndef HLINT
 -- | inlineable 'unsafePerformIO'
 inlinePerformIO :: IO a -> a
 inlinePerformIO (IO m) = case m realWorld# of
   (# _, r #) -> r
 {-# INLINE inlinePerformIO #-}
-#endif
 
 -------------------------------------------------------------------------------
 -- Cache
@@ -396,7 +391,7 @@ hitTest a b = Oracle $ \(c :: c) ->
 
 
 biplateData :: forall f s a. (Applicative f, Data s) => (forall c. Typeable c => c -> Answer c a) -> (a -> f a) -> s -> f s
-biplateData o f a0 = go2 a0 where
+biplateData o f = go2 where
   go :: Data d => d -> f d
   go s = gfoldl (\x y -> x <*> go2 y) pure s
   go2 :: Data d => d -> f d
@@ -407,7 +402,7 @@ biplateData o f a0 = go2 a0 where
 {-# INLINE biplateData #-}
 
 uniplateData :: forall f s a. (Applicative f, Data s) => (forall c. Typeable c => c -> Answer c a) -> (a -> f a) -> s -> f s
-uniplateData o f a0 = go a0 where
+uniplateData o f = go where
   go :: Data d => d -> f d
   go s = gfoldl (\x y -> x <*> go2 y) pure s
   go2 :: Data d => d -> f d
