@@ -1,16 +1,12 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleInstances #-}
 #if defined(__GLASGOW_HASKELL__)
 {-# LANGUAGE Trustworthy #-}
 #endif
 {-# LANGUAGE Rank2Types #-}
 
-#if __GLASGOW_HASKELL__ >= 800
-{-# OPTIONS_GHC -Wno-trustworthy-safe #-}
-#endif
+#include "lens-common.h"
 
-#ifndef MIN_VERSION_template_haskell
-#define MIN_VERSION_template_haskell(x,y,z) 1
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.TH.Lens
@@ -66,14 +62,12 @@ module Language.Haskell.TH.Lens
   -- ** FieldPat Lenses
   , fieldPatName
   , fieldPatPattern
-#if MIN_VERSION_template_haskell(2,9,0)
   -- ** TySynEqn Lenses
 # if MIN_VERSION_template_haskell(2,15,0)
   , tySynEqnLHS
 # endif
   , tySynEqnPatterns
   , tySynEqnResult
-#endif
 #if MIN_VERSION_template_haskell(2,11,0)
   -- ** InjectivityAnn Lenses
   , injectivityAnnOutput
@@ -115,17 +109,13 @@ module Language.Haskell.TH.Lens
   , _InstanceD
   , _SigD
   , _ForeignD
-#if MIN_VERSION_template_haskell(2,8,0)
   , _InfixD
-#endif
   , _PragmaD
   , _DataInstD
   , _NewtypeInstD
   , _TySynInstD
-#if MIN_VERSION_template_haskell(2,9,0)
   , _ClosedTypeFamilyD
   , _RoleAnnotD
-#endif
 #if MIN_VERSION_template_haskell(2,10,0)
   , _StandaloneDerivD
   , _DefaultSigD
@@ -206,12 +196,9 @@ module Language.Haskell.TH.Lens
   -- ** Pragma Prisms
   , _InlineP
   , _SpecialiseP
-#if MIN_VERSION_template_haskell(2,8,0)
   , _SpecialiseInstP
   , _RuleP
-#if MIN_VERSION_template_haskell(2,9,0)
   , _AnnP
-#endif
 #if MIN_VERSION_template_haskell(2,10,0)
   , _LineP
 #endif
@@ -232,13 +219,10 @@ module Language.Haskell.TH.Lens
   -- ** RuleBndr Prisms
   , _RuleVar
   , _TypedRuleVar
-#endif
-#if MIN_VERSION_template_haskell(2,9,0)
   -- ** AnnTarget Prisms
   , _ModuleAnnotation
   , _TypeAnnotation
   , _ValueAnnotation
-#endif
   -- ** FunDep Prisms TODO make a lens
   , _FunDep
 #if !(MIN_VERSION_template_haskell(2,13,0))
@@ -262,18 +246,14 @@ module Language.Haskell.TH.Lens
   , _UInfixE
   , _ParensE
   , _LamE
-#if MIN_VERSION_template_haskell(2,8,0)
   , _LamCaseE
-#endif
   , _TupE
   , _UnboxedTupE
 #if MIN_VERSION_template_haskell(2,12,0)
   , _UnboxedSumE
 #endif
   , _CondE
-#if MIN_VERSION_template_haskell(2,8,0)
   , _MultiIfE
-#endif
   , _LetE
   , _CaseE
   , _DoE
@@ -327,6 +307,9 @@ module Language.Haskell.TH.Lens
   , _StringPrimL
 #if MIN_VERSION_template_haskell(2,11,0)
   , _CharPrimL
+#endif
+#if MIN_VERSION_template_haskell(2,16,0)
+  , _BytesPrimL
 #endif
   -- ** Pat Prisms
   , _LitP
@@ -354,9 +337,7 @@ module Language.Haskell.TH.Lens
   , _SigT
   , _VarT
   , _ConT
-#if MIN_VERSION_template_haskell(2,8,0)
   , _PromotedT
-#endif
   , _TupleT
   , _UnboxedTupleT
 #if MIN_VERSION_template_haskell(2,12,0)
@@ -367,14 +348,12 @@ module Language.Haskell.TH.Lens
   , _EqualityT
 #endif
   , _ListT
-#if MIN_VERSION_template_haskell(2,8,0)
   , _PromotedTupleT
   , _PromotedNilT
   , _PromotedConsT
   , _StarT
   , _ConstraintT
   , _LitT
-#endif
 #if MIN_VERSION_template_haskell(2,11,0)
   , _InfixT
   , _UInfixT
@@ -384,6 +363,17 @@ module Language.Haskell.TH.Lens
 #if MIN_VERSION_template_haskell(2,15,0)
   , _AppKindT
   , _ImplicitParamT
+#endif
+#if MIN_VERSION_template_haskell(2,16,0)
+  , _ForallVisT
+#endif
+#if MIN_VERSION_template_haskell(2,17,0)
+  , _MulArrowT
+#endif
+#if MIN_VERSION_template_haskell(2,17,0)
+  -- ** Specificity Prisms
+  , _SpecifiedSpec
+  , _InferredSpec
 #endif
   -- ** TyVarBndr Prisms
   , _PlainTV
@@ -394,23 +384,19 @@ module Language.Haskell.TH.Lens
   , _KindSig
   , _TyVarSig
 #endif
-#if MIN_VERSION_template_haskell(2,8,0)
   -- ** TyLit Prisms
   , _NumTyLit
   , _StrTyLit
-#endif
 #if !MIN_VERSION_template_haskell(2,10,0)
   -- ** Pred Prisms
   , _ClassP
   , _EqualP
 #endif
-#if MIN_VERSION_template_haskell(2,9,0)
   -- ** Role Prisms
   , _NominalR
   , _RepresentationalR
   , _PhantomR
   , _InferR
-#endif
 #if MIN_VERSION_template_haskell(2,12,0)
   -- ** DerivStrategy Prisms
   , _StockStrategy
@@ -429,16 +415,17 @@ import Control.Lens.Lens
 import Control.Lens.Prism
 import Control.Lens.Tuple
 import Control.Lens.Traversal
-import Data.Map as Map hiding (toList,map)
+import qualified Data.Map as Map
+import Data.Map (Map)
 import Data.Maybe (fromMaybe)
 import Data.Monoid
-import Data.Set as Set hiding (toList,map)
+import qualified Data.Set as Set
+import Data.Set (Set)
 import Data.Set.Lens
 import Language.Haskell.TH
+import Language.Haskell.TH.Datatype.TyVarBndr
 import Language.Haskell.TH.Syntax
-#if MIN_VERSION_template_haskell(2,8,0)
 import Data.Word
-#endif
 #if MIN_VERSION_template_haskell(2,15,0)
 import Control.Lens.Internal.TH (unfoldType)
 import Data.Foldable as F (foldl')
@@ -450,9 +437,8 @@ class HasName t where
   -- | Extract (or modify) the 'Name' of something
   name :: Lens' t Name
 
-instance HasName TyVarBndr where
-  name f (PlainTV n) = PlainTV <$> f n
-  name f (KindedTV n k) = (`KindedTV` k) <$> f n
+instance HasName (TyVarBndr_ flag) where
+  name = traverseTVName
 
 instance HasName Name where
   name = id
@@ -477,11 +463,9 @@ instance HasName Foreign where
   name f (ExportF cc str n ty) =
     (\n' -> ExportF cc str n' ty) <$> f n
 
-#if MIN_VERSION_template_haskell(2,8,0)
 instance HasName RuleBndr where
   name f (RuleVar n) = RuleVar <$> f n
   name f (TypedRuleVar n ty) = (`TypedRuleVar` ty) <$> f n
-#endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
 instance HasName TypeFamilyHead where
@@ -517,20 +501,17 @@ instance HasTypes Foreign where
   types f (ImportF cc saf str n t) = ImportF cc saf str n <$> types f t
   types f (ExportF cc     str n t) = ExportF cc     str n <$> types f t
 
-#if MIN_VERSION_template_haskell(2,9,0)
 instance HasTypes TySynEqn where
 #if MIN_VERSION_template_haskell(2,15,0)
   types f (TySynEqn mtvbs lhs rhs) = TySynEqn <$> traverse (traverse go) mtvbs
                                               <*> types f lhs
                                               <*> types f rhs
     where
-      go tvb@PlainTV{} = pure tvb
-      go (KindedTV n k)  = KindedTV n <$> f k
+      go = traverseTVKind f
 #else
   types f (TySynEqn lhss rhs) = TySynEqn <$> traverse (types f) lhss
                                          <*> types f rhs
 #endif
-#endif
 
 instance HasTypes t => HasTypes [t] where
   types = traverse . types
@@ -542,7 +523,7 @@ class HasTypeVars t where
   -- the 'Traversal' laws, when in doubt generate your names with 'newName'.
   typeVarsEx :: Set Name -> Traversal' t Name
 
-instance HasTypeVars TyVarBndr where
+instance HasTypeVars (TyVarBndr_ flag) where
   typeVarsEx s f b
     | s^.contains (b^.name) = pure b
     | otherwise             = name f b
@@ -555,14 +536,25 @@ instance HasTypeVars Name where
 instance HasTypeVars Type where
   typeVarsEx s f (VarT n)             = VarT <$> typeVarsEx s f n
   typeVarsEx s f (AppT l r)           = AppT <$> typeVarsEx s f l <*> typeVarsEx s f r
-#if MIN_VERSION_template_haskell(2,8,0)
-  typeVarsEx s f (SigT t k)           = SigT <$> typeVarsEx s f t
-                                             <*> typeVarsEx s f k
-#else
-  typeVarsEx s f (SigT t k)           = (`SigT` k) <$> typeVarsEx s f t
-#endif
   typeVarsEx s f (ForallT bs ctx ty)  = ForallT bs <$> typeVarsEx s' f ctx <*> typeVarsEx s' f ty
        where s' = s `Set.union` setOf typeVars bs
+  typeVarsEx _ _ t@ConT{}             = pure t
+  typeVarsEx _ _ t@TupleT{}           = pure t
+  typeVarsEx _ _ t@ListT{}            = pure t
+  typeVarsEx _ _ t@ArrowT{}           = pure t
+  typeVarsEx _ _ t@UnboxedTupleT{}    = pure t
+  typeVarsEx s f (SigT t k)           = SigT <$> typeVarsEx s f t
+                                             <*> typeVarsEx s f k
+  typeVarsEx _ _ t@PromotedT{}        = pure t
+  typeVarsEx _ _ t@PromotedTupleT{}   = pure t
+  typeVarsEx _ _ t@PromotedNilT{}     = pure t
+  typeVarsEx _ _ t@PromotedConsT{}    = pure t
+  typeVarsEx _ _ t@StarT{}            = pure t
+  typeVarsEx _ _ t@ConstraintT{}      = pure t
+  typeVarsEx _ _ t@LitT{}             = pure t
+#if MIN_VERSION_template_haskell(2,10,0)
+  typeVarsEx _ _ t@EqualityT{}        = pure t
+#endif
 #if MIN_VERSION_template_haskell(2,11,0)
   typeVarsEx s f (InfixT  t1 n t2)    = InfixT  <$> typeVarsEx s f t1
                                                 <*> pure n
@@ -571,13 +563,23 @@ instance HasTypeVars Type where
                                                 <*> pure n
                                                 <*> typeVarsEx s f t2
   typeVarsEx s f (ParensT t)          = ParensT <$> typeVarsEx s f t
+  typeVarsEx _ _ t@WildCardT{}        = pure t
+#endif
+#if MIN_VERSION_template_haskell(2,12,0)
+  typeVarsEx _ _ t@UnboxedSumT{}      = pure t
 #endif
 #if MIN_VERSION_template_haskell(2,15,0)
   typeVarsEx s f (AppKindT t k)       = AppKindT <$> typeVarsEx s f t
                                                  <*> typeVarsEx s f k
   typeVarsEx s f (ImplicitParamT n t) = ImplicitParamT n <$> typeVarsEx s f t
 #endif
-  typeVarsEx _ _ t                    = pure t
+#if MIN_VERSION_template_haskell(2,16,0)
+  typeVarsEx s f (ForallVisT bs ty)   = ForallVisT bs <$> typeVarsEx s' f ty
+       where s' = s `Set.union` setOf typeVars bs
+#endif
+#if MIN_VERSION_template_haskell(2,17,0)
+  typeVarsEx _ _ t@MulArrowT{}        = pure t
+#endif
 
 #if !MIN_VERSION_template_haskell(2,10,0)
 instance HasTypeVars Pred where
@@ -624,22 +626,44 @@ instance SubstType Type where
   substType m t@(VarT n)           = fromMaybe t (m^.at n)
   substType m (ForallT bs ctx ty)  = ForallT bs (substType m' ctx) (substType m' ty)
     where m' = foldrOf typeVars Map.delete m bs
-#if MIN_VERSION_template_haskell(2,8,0)
-  substType m (SigT t k)           = SigT (substType m t) (substType m k)
-#else
-  substType m (SigT t k)           = SigT (substType m t) k
-#endif
+  substType _ t@ConT{}             = t
+  substType _ t@TupleT{}           = t
+  substType _ t@ListT{}            = t
+  substType _ t@ArrowT{}           = t
+  substType _ t@UnboxedTupleT{}    = t
   substType m (AppT l r)           = AppT (substType m l) (substType m r)
+  substType m (SigT t k)           = SigT (substType m t)
+                                          (substType m k)
+  substType _ t@PromotedT{}        = t
+  substType _ t@PromotedTupleT{}   = t
+  substType _ t@PromotedNilT{}     = t
+  substType _ t@PromotedConsT{}    = t
+  substType _ t@StarT{}            = t
+  substType _ t@ConstraintT{}      = t
+  substType _ t@LitT{}             = t
+#if MIN_VERSION_template_haskell(2,10,0)
+  substType _ t@EqualityT{}        = t
+#endif
 #if MIN_VERSION_template_haskell(2,11,0)
   substType m (InfixT  t1 n t2)    = InfixT  (substType m t1) n (substType m t2)
   substType m (UInfixT t1 n t2)    = UInfixT (substType m t1) n (substType m t2)
   substType m (ParensT t)          = ParensT (substType m t)
+  substType _ t@WildCardT{}        = t
+#endif
+#if MIN_VERSION_template_haskell(2,12,0)
+  substType _ t@UnboxedSumT{}      = t
 #endif
 #if MIN_VERSION_template_haskell(2,15,0)
   substType m (AppKindT t k)       = AppKindT (substType m t) (substType m k)
   substType m (ImplicitParamT n t) = ImplicitParamT n (substType m t)
 #endif
-  substType _ t                   = t
+#if MIN_VERSION_template_haskell(2,16,0)
+  substType m (ForallVisT bs ty)   = ForallVisT bs (substType m' ty)
+    where m' = foldrOf typeVars Map.delete m bs
+#endif
+#if MIN_VERSION_template_haskell(2,17,0)
+  substType _ t@MulArrowT{}        = t
+#endif
 
 instance SubstType t => SubstType [t] where
   substType = map . substType
@@ -697,13 +721,15 @@ conNamedFields :: Traversal' Con
 #else
                                  VarStrictType
 #endif
-conNamedFields f (RecC n fs) = RecC n <$> traverse f fs
+conNamedFields _ c@NormalC{}      = pure c
+conNamedFields _ c@InfixC{}       = pure c
+conNamedFields f (RecC n fs)      = RecC n <$> traverse f fs
 conNamedFields f (ForallC a b fs) = ForallC a b <$> conNamedFields f fs
 #if MIN_VERSION_template_haskell(2,11,0)
+conNamedFields _ c@GadtC{}        = pure c
 conNamedFields f (RecGadtC ns argTys retTy) =
   RecGadtC ns <$> traverse f argTys <*> pure retTy
 #endif
-conNamedFields _ c = pure c
 
 -- Lenses and Prisms
 locFileName :: Lens' Loc String
@@ -804,7 +830,7 @@ typeFamilyHeadName = lens g s where
   g (TypeFamilyHead n _    _  _ )   = n
   s (TypeFamilyHead _ tvbs rs ia) n = TypeFamilyHead n tvbs rs ia
 
-typeFamilyHeadTyVarBndrs :: Lens' TypeFamilyHead [TyVarBndr]
+typeFamilyHeadTyVarBndrs :: Lens' TypeFamilyHead [TyVarBndrUnit]
 typeFamilyHeadTyVarBndrs = lens g s where
   g (TypeFamilyHead _ tvbs _  _ )      = tvbs
   s (TypeFamilyHead n _    rs ia) tvbs = TypeFamilyHead n tvbs rs ia
@@ -842,11 +868,7 @@ derivClauseCxt = lens g s where
   s (DerivClause mds _) = DerivClause mds
 #endif
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _ClassI :: Prism' Info (Dec, [InstanceDec])
-#else
-_ClassI :: Prism' Info (Dec, [Dec])
-#endif
 _ClassI
   = prism' reviewer remitter
   where
@@ -869,11 +891,7 @@ _ClassOpI
       remitter (ClassOpI x y z) = Just (x, y, z)
       remitter _ = Nothing
 #else
-# if MIN_VERSION_template_haskell(2,8,0)
 _ClassOpI :: Prism' Info (Name, Type, ParentName, Fixity)
-# else
-_ClassOpI :: Prism' Info (Name, Type, Name, Fixity)
-# endif
 _ClassOpI
   = prism' reviewer remitter
   where
@@ -902,11 +920,7 @@ _FamilyI
       remitter (FamilyI x y) = Just (x, y)
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _PrimTyConI :: Prism' Info (Name, Arity, Unlifted)
-#else
-_PrimTyConI :: Prism' Info (Name, Int, Bool)
-#endif
 _PrimTyConI
   = prism' reviewer remitter
   where
@@ -929,11 +943,7 @@ _DataConI
       remitter (DataConI x y z) = Just (x, y, z)
       remitter _ = Nothing
 #else
-# if MIN_VERSION_template_haskell(2,8,0)
 _DataConI :: Prism' Info (Name, Type, ParentName, Fixity)
-# else
-_DataConI :: Prism' Info (Name, Type, Name, Fixity)
-# endif
 _DataConI
   = prism' reviewer remitter
   where
@@ -999,7 +1009,7 @@ _ValD
       remitter (ValD x y z) = Just (x, y, z)
       remitter _ = Nothing
 
-_TySynD :: Prism' Dec (Name, [TyVarBndr], Type)
+_TySynD :: Prism' Dec (Name, [TyVarBndrUnit], Type)
 _TySynD
   = prism' reviewer remitter
   where
@@ -1007,7 +1017,7 @@ _TySynD
       remitter (TySynD x y z) = Just (x, y, z)
       remitter _ = Nothing
 
-_ClassD :: Prism' Dec (Cxt, Name, [TyVarBndr], [FunDep], [Dec])
+_ClassD :: Prism' Dec (Cxt, Name, [TyVarBndrUnit], [FunDep], [Dec])
 _ClassD
   = prism' reviewer remitter
   where
@@ -1083,7 +1093,6 @@ _ForeignD
       remitter (ForeignD x) = Just x
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _InfixD :: Prism' Dec (Fixity, Name)
 _InfixD
   = prism' reviewer remitter
@@ -1091,7 +1100,6 @@ _InfixD
       reviewer (x, y) = InfixD x y
       remitter (InfixD x y) = Just (x, y)
       remitter _ = Nothing
-#endif
 
 _PragmaD :: Prism' Dec Pragma
 _PragmaD
@@ -1115,7 +1123,7 @@ _TySynInstD
       reviewer = TySynInstD
       remitter (TySynInstD x) = Just x
       remitter _ = Nothing
-#elif MIN_VERSION_template_haskell(2,9,0)
+#else
 _TySynInstD :: Prism' Dec (Name, TySynEqn)
 _TySynInstD
   = prism' reviewer remitter
@@ -1123,17 +1131,8 @@ _TySynInstD
       reviewer (x, y) = TySynInstD x y
       remitter (TySynInstD x y) = Just (x, y)
       remitter _ = Nothing
-#else
-_TySynInstD :: Prism' Dec (Name, [Type], Type)
-_TySynInstD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z) = TySynInstD x y z
-      remitter (TySynInstD x y z) = Just (x, y, z)
-      remitter _ = Nothing
 #endif
 
-#if MIN_VERSION_template_haskell(2,9,0)
 _RoleAnnotD :: Prism' Dec (Name, [Role])
 _RoleAnnotD
   = prism' reviewer remitter
@@ -1141,7 +1140,6 @@ _RoleAnnotD
       reviewer (x, y) = RoleAnnotD x y
       remitter (RoleAnnotD x y) = Just (x, y)
       remitter _ = Nothing
-#endif
 
 -- |
 -- @
@@ -1184,13 +1182,13 @@ type DataPrism' tys cons = Prism' Dec (Cxt, Name, tys, Maybe Kind, cons, Cxt)
 
 -- |
 -- @
--- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Maybe' ['TyVarBndr'], 'Type', 'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.15+
--- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],            'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.12 through 2.14
--- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],            'Maybe' 'Kind', ['Con'], 'Cxt')           -- template-haskell-2.11
--- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                        ['Con'], ['Name'])        -- Earlier versions
+-- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Maybe' ['TyVarBndrUnit'], 'Type', 'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.15+
+-- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.12 through 2.14
+-- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                'Maybe' 'Kind', ['Con'], 'Cxt')           -- template-haskell-2.11
+-- _DataInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                            ['Con'], ['Name'])        -- Earlier versions
 -- @
 #if MIN_VERSION_template_haskell(2,15,0)
-_DataInstD :: Prism' Dec (Cxt, Maybe [TyVarBndr], Type, Maybe Kind, [Con], [DerivClause])
+_DataInstD :: Prism' Dec (Cxt, Maybe [TyVarBndrUnit], Type, Maybe Kind, [Con], [DerivClause])
 _DataInstD
   = prism' reviewer remitter
   where
@@ -1217,13 +1215,13 @@ _DataInstD
 
 -- |
 -- @
--- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Maybe' ['TyVarBndr'], 'Type', 'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.15+
--- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],            'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.12 through 2.14
--- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],            'Maybe' 'Kind', 'Con', 'Cxt')           -- template-haskell-2.11
--- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                        'Con', ['Name'])        -- Earlier versions
+-- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Maybe' ['TyVarBndrUnit'], 'Type', 'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.15+
+-- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.12 through 2.14
+-- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                'Maybe' 'Kind', 'Con', 'Cxt')           -- template-haskell-2.11
+-- _NewtypeInstD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                            'Con', ['Name'])        -- Earlier versions
 -- @
 #if MIN_VERSION_template_haskell(2,15,0)
-_NewtypeInstD :: Prism' Dec (Cxt, Maybe [TyVarBndr], Type, Maybe Kind, Con, [DerivClause])
+_NewtypeInstD :: Prism' Dec (Cxt, Maybe [TyVarBndrUnit], Type, Maybe Kind, Con, [DerivClause])
 _NewtypeInstD
   = prism' reviewer remitter
   where
@@ -1256,8 +1254,8 @@ _ClosedTypeFamilyD
       reviewer (x, y) = ClosedTypeFamilyD x y
       remitter (ClosedTypeFamilyD x y) = Just (x, y)
       remitter _ = Nothing
-#elif MIN_VERSION_template_haskell(2,9,0)
-_ClosedTypeFamilyD :: Prism' Dec (Name, [TyVarBndr], Maybe Kind, [TySynEqn])
+#else
+_ClosedTypeFamilyD :: Prism' Dec (Name, [TyVarBndrUnit], Maybe Kind, [TySynEqn])
 _ClosedTypeFamilyD
   = prism' reviewer remitter
   where
@@ -1268,12 +1266,12 @@ _ClosedTypeFamilyD
 
 -- |
 -- @
--- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['TyVarBndr'], 'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.12+
--- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],      'Maybe' 'Kind', ['Con'], 'Cxt')           -- template-haskell-2.11
--- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                  ['Con'], ['Name'])        -- Earlier versions
+-- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['TyVarBndrUnit'], 'Maybe' 'Kind', ['Con'], ['DerivClause']) -- template-haskell-2.12+
+-- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],          'Maybe' 'Kind', ['Con'], 'Cxt')           -- template-haskell-2.11
+-- _DataD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                      ['Con'], ['Name'])        -- Earlier versions
 -- @
 #if MIN_VERSION_template_haskell(2,11,0)
-_DataD :: DataPrism' [TyVarBndr] [Con]
+_DataD :: DataPrism' [TyVarBndrUnit] [Con]
 _DataD
   = prism' reviewer remitter
   where
@@ -1281,7 +1279,7 @@ _DataD
       remitter (DataD x y z w u v) = Just (x, y, z, w, u, v)
       remitter _ = Nothing
 #else
-_DataD :: Prism' Dec (Cxt, Name, [TyVarBndr], [Con], [Name])
+_DataD :: Prism' Dec (Cxt, Name, [TyVarBndrUnit], [Con], [Name])
 _DataD
   = prism' reviewer remitter
   where
@@ -1292,12 +1290,12 @@ _DataD
 
 -- |
 -- @
--- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['TyVarBndr'], 'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.12+
--- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],      'Maybe' 'Kind', 'Con', 'Cxt')           -- template-haskell-2.11
--- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                  'Con', ['Name'])        -- Earlier versions
+-- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['TyVarBndrUnit'], 'Maybe' 'Kind', 'Con', ['DerivClause']) -- template-haskell-2.12+
+-- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],          'Maybe' 'Kind', 'Con', 'Cxt')           -- template-haskell-2.11
+-- _NewtypeD :: 'Prism'' 'Dec' ('Cxt', 'Name', ['Type'],                      'Con', ['Name'])        -- Earlier versions
 -- @
 #if MIN_VERSION_template_haskell(2,11,0)
-_NewtypeD :: DataPrism' [TyVarBndr] Con
+_NewtypeD :: DataPrism' [TyVarBndrUnit] Con
 _NewtypeD
   = prism' reviewer remitter
   where
@@ -1305,7 +1303,7 @@ _NewtypeD
       remitter (NewtypeD x y z w u v) = Just (x, y, z, w, u, v)
       remitter _ = Nothing
 #else
-_NewtypeD :: Prism' Dec (Cxt, Name, [TyVarBndr], Con, [Name])
+_NewtypeD :: Prism' Dec (Cxt, Name, [TyVarBndrUnit], Con, [Name])
 _NewtypeD
   = prism' reviewer remitter
   where
@@ -1315,7 +1313,7 @@ _NewtypeD
 #endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
-_DataFamilyD :: Prism' Dec (Name, [TyVarBndr], Maybe Kind)
+_DataFamilyD :: Prism' Dec (Name, [TyVarBndrUnit], Maybe Kind)
 _DataFamilyD
   = prism' reviewer remitter
   where
@@ -1331,7 +1329,7 @@ _OpenTypeFamilyD
       remitter (OpenTypeFamilyD x) = Just x
       remitter _ = Nothing
 #else
-_FamilyD :: Prism' Dec (FamFlavour, Name, [TyVarBndr], Maybe Kind)
+_FamilyD :: Prism' Dec (FamFlavour, Name, [TyVarBndrUnit], Maybe Kind)
 _FamilyD
   = prism' reviewer remitter
   where
@@ -1477,7 +1475,7 @@ _InfixC
       remitter (InfixC x y z) = Just (x, y, z)
       remitter _ = Nothing
 
-_ForallC :: Prism' Con ([TyVarBndr], Cxt, Con)
+_ForallC :: Prism' Con ([TyVarBndrSpec], Cxt, Con)
 _ForallC
   = prism' reviewer remitter
   where
@@ -1683,12 +1681,6 @@ _Interruptible
       remitter Interruptible = Just ()
       remitter _ = Nothing
 
--- |
--- @
--- _InlineP :: 'Prism'' 'Pragma' ('Name', 'Inline', 'RuleMatch', 'Phases') -- template-haskell-2.8+
--- _InlineP :: 'Prism'' 'Pragma' ('Name', 'Inline')                    -- Earlier versions
--- @
-#if MIN_VERSION_template_haskell(2,8,0)
 _InlineP :: Prism' Pragma (Name, Inline, RuleMatch, Phases)
 _InlineP
   = prism' reviewer remitter
@@ -1696,22 +1688,7 @@ _InlineP
       reviewer (x, y, z, w) = InlineP x y z w
       remitter (InlineP x y z w) = Just (x, y, z, w)
       remitter _ = Nothing
-#else
-_InlineP :: Prism' Pragma (Name, InlineSpec)
-_InlineP
-  = prism' reviewer remitter
-  where
-      reviewer (x, y) = InlineP x y
-      remitter (InlineP x y) = Just (x, y)
-      remitter _ = Nothing
-#endif
 
--- |
--- @
--- _SpecialiseP :: 'Prism'' 'Pragma' ('Name', 'Type', 'Maybe' 'Inline', 'Phases') -- template-haskell-2.8+
--- _SpecialiseP :: 'Prism'' 'Pragma' ('Name', 'Type', 'Maybe' 'InlineSpec')     -- Earlier versions
--- @
-#if MIN_VERSION_template_haskell(2,8,0)
 _SpecialiseP :: Prism' Pragma (Name, Type, Maybe Inline, Phases)
 _SpecialiseP
   = prism' reviewer remitter
@@ -1719,19 +1696,9 @@ _SpecialiseP
       reviewer (x, y, z, w) = SpecialiseP x y z w
       remitter (SpecialiseP x y z w) = Just (x, y, z, w)
       remitter _ = Nothing
-#else
-_SpecialiseP :: Prism' Pragma (Name, Type, Maybe InlineSpec)
-_SpecialiseP
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z) = SpecialiseP x y z
-      remitter (SpecialiseP x y z) = Just (x, y, z)
-      remitter _ = Nothing
-#endif
 
 -- TODO add lenses for InlineSpec
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _SpecialiseInstP :: Prism' Pragma Type
 _SpecialiseInstP
   = prism' reviewer remitter
@@ -1742,11 +1709,11 @@ _SpecialiseInstP
 
 -- |
 -- @
--- _RuleP :: 'Prism'' 'Pragma' ('String', 'Maybe' ['TyVarBndr'], ['RuleBndr'], 'Exp', 'Exp', 'Phases') -- template-haskell-2.15+
--- _RuleP :: 'Prism'' 'Pragma' ('String',                    ['RuleBndr'], 'Exp', 'Exp', 'Phases') -- Earlier versions
+-- _RuleP :: 'Prism'' 'Pragma' ('String', 'Maybe' ['TyVarBndrUnit'], ['RuleBndr'], 'Exp', 'Exp', 'Phases') -- template-haskell-2.15+
+-- _RuleP :: 'Prism'' 'Pragma' ('String',                        ['RuleBndr'], 'Exp', 'Exp', 'Phases') -- Earlier versions
 -- @
 #if MIN_VERSION_template_haskell(2,15,0)
-_RuleP :: Prism' Pragma (String, Maybe [TyVarBndr], [RuleBndr], Exp, Exp, Phases)
+_RuleP :: Prism' Pragma (String, Maybe [TyVarBndrUnit], [RuleBndr], Exp, Exp, Phases)
 _RuleP
   = prism' reviewer remitter
   where
@@ -1763,7 +1730,6 @@ _RuleP
       remitter _ = Nothing
 #endif
 
-#if MIN_VERSION_template_haskell(2,9,0)
 _AnnP :: Prism' Pragma (AnnTarget, Exp)
 _AnnP
   = prism' reviewer remitter
@@ -1771,7 +1737,6 @@ _AnnP
       reviewer (x, y) = AnnP x y
       remitter (AnnP x y) = Just (x, y)
       remitter _ = Nothing
-#endif
 
 #if MIN_VERSION_template_haskell(2,10,0)
 _LineP :: Prism' Pragma (Int, String)
@@ -1872,9 +1837,7 @@ _TypedRuleVar
       reviewer (x, y) = TypedRuleVar x y
       remitter (TypedRuleVar x y) = Just (x, y)
       remitter _ = Nothing
-#endif
 
-#if MIN_VERSION_template_haskell(2,9,0)
 _ModuleAnnotation :: Prism' AnnTarget ()
 _ModuleAnnotation
   = prism' reviewer remitter
@@ -1898,7 +1861,6 @@ _ValueAnnotation
       reviewer = ValueAnnotation
       remitter (ValueAnnotation x) = Just x
       remitter _ = Nothing
-#endif
 
 _FunDep :: Iso' FunDep ([Name], [Name])
 _FunDep
@@ -1942,7 +1904,7 @@ tySynEqnResult :: Lens' TySynEqn Type
 tySynEqnResult = lens g s where
    g (TySynEqn _     _   rhs) = rhs
    s (TySynEqn mtvbs lhs _)   = TySynEqn mtvbs lhs
-#elif MIN_VERSION_template_haskell(2,9,0)
+#else
 tySynEqnPatterns :: Lens' TySynEqn [Type]
 tySynEqnPatterns = lens g s where
    g (TySynEqn xs _)    = xs
@@ -2052,7 +2014,6 @@ _LamE
       remitter (LamE x y) = Just (x, y)
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _LamCaseE :: Prism' Exp [Match]
 _LamCaseE
   = prism' reviewer remitter
@@ -2060,9 +2021,17 @@ _LamCaseE
       reviewer = LamCaseE
       remitter (LamCaseE x) = Just x
       remitter _ = Nothing
-#endif
 
+-- |
+-- @
+-- _TupE :: 'Prism'' 'Exp' ['Maybe' 'Exp'] -- template-haskell-2.16+
+-- _TupE :: 'Prism'' 'Exp' ['Exp']       -- Earlier versions
+-- @
+#if MIN_VERSION_template_haskell(2,16,0)
+_TupE :: Prism' Exp [Maybe Exp]
+#else
 _TupE :: Prism' Exp [Exp]
+#endif
 _TupE
   = prism' reviewer remitter
   where
@@ -2070,7 +2039,16 @@ _TupE
       remitter (TupE x) = Just x
       remitter _ = Nothing
 
+-- |
+-- @
+-- _UnboxedTupE :: 'Prism'' 'Exp' ['Maybe' 'Exp'] -- template-haskell-2.16+
+-- _UnboxedTupE :: 'Prism'' 'Exp' ['Exp']       -- Earlier versions
+-- @
+#if MIN_VERSION_template_haskell(2,16,0)
+_UnboxedTupE :: Prism' Exp [Maybe Exp]
+#else
 _UnboxedTupE :: Prism' Exp [Exp]
+#endif
 _UnboxedTupE
   = prism' reviewer remitter
   where
@@ -2096,7 +2074,6 @@ _CondE
       remitter (CondE x y z) = Just (x, y, z)
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _MultiIfE :: Prism' Exp [(Guard, Exp)]
 _MultiIfE
   = prism' reviewer remitter
@@ -2104,7 +2081,6 @@ _MultiIfE
       reviewer = MultiIfE
       remitter (MultiIfE x) = Just x
       remitter _ = Nothing
-#endif
 
 _LetE :: Prism' Exp ([Dec], Exp)
 _LetE
@@ -2122,6 +2098,20 @@ _CaseE
       remitter (CaseE x y) = Just (x, y)
       remitter _ = Nothing
 
+-- |
+-- @
+-- _DoE :: 'Prism'' 'Exp' ('Maybe' 'ModName', ['Stmt']) -- template-haskell-2.17+
+-- _DoE :: 'Prism'' 'Exp' ['Stmt']                  -- Earlier versions
+-- @
+# if MIN_VERSION_template_haskell(2,17,0)
+_DoE :: Prism' Exp (Maybe ModName, [Stmt])
+_DoE
+  = prism' reviewer remitter
+  where
+      reviewer (x, y) = DoE x y
+      remitter (DoE x y) = Just (x, y)
+      remitter _ = Nothing
+# else
 _DoE :: Prism' Exp [Stmt]
 _DoE
   = prism' reviewer remitter
@@ -2129,6 +2119,7 @@ _DoE
       reviewer = DoE
       remitter (DoE x) = Just x
       remitter _ = Nothing
+# endif
 
 _CompE :: Prism' Exp [Stmt]
 _CompE
@@ -2209,6 +2200,20 @@ _LabelE
 #endif
 
 #if MIN_VERSION_template_haskell(2,15,0)
+-- |
+-- @
+-- _MDoE :: 'Prism'' 'Exp' ('Maybe' 'ModName', ['Stmt']) -- template-haskell-2.17+
+-- _MDoE :: 'Prism'' 'Exp' ['Stmt']                  -- Earlier versions
+-- @
+# if MIN_VERSION_template_haskell(2,17,0)
+_MDoE :: Prism' Exp (Maybe ModName, [Stmt])
+_MDoE
+  = prism' reviewer remitter
+  where
+      reviewer (x, y) = MDoE x y
+      remitter (MDoE x y) = Just (x, y)
+      remitter _ = Nothing
+# else
 _MDoE :: Prism' Exp [Stmt]
 _MDoE
   = prism' reviewer remitter
@@ -2216,6 +2221,7 @@ _MDoE
       reviewer = MDoE
       remitter (MDoE x) = Just x
       remitter _ = Nothing
+# endif
 
 _ImplicitParamVarE :: Prism' Exp String
 _ImplicitParamVarE
@@ -2396,7 +2402,6 @@ _DoublePrimL
       remitter (DoublePrimL x) = Just x
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _StringPrimL :: Prism' Lit [Word8]
 _StringPrimL
   = prism' reviewer remitter
@@ -2404,15 +2409,6 @@ _StringPrimL
       reviewer = StringPrimL
       remitter (StringPrimL x) = Just x
       remitter _ = Nothing
-#else
-_StringPrimL :: Prism' Lit String
-_StringPrimL
-  = prism' reviewer remitter
-  where
-      reviewer = StringPrimL
-      remitter (StringPrimL x) = Just x
-      remitter _ = Nothing
-#endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
 _CharPrimL :: Prism' Lit Char
@@ -2424,6 +2420,16 @@ _CharPrimL
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,16,0)
+_BytesPrimL :: Prism' Lit Bytes
+_BytesPrimL
+  = prism' reviewer remitter
+  where
+      reviewer = BytesPrimL
+      remitter (BytesPrimL x) = Just x
+      remitter _ = Nothing
+#endif
+
 _LitP :: Prism' Pat Lit
 _LitP
   = prism' reviewer remitter
@@ -2562,7 +2568,7 @@ _ViewP
       remitter (ViewP x y) = Just (x, y)
       remitter _ = Nothing
 
-_ForallT :: Prism' Type ([TyVarBndr], Cxt, Type)
+_ForallT :: Prism' Type ([TyVarBndrSpec], Cxt, Type)
 _ForallT
   = prism' reviewer remitter
   where
@@ -2602,7 +2608,6 @@ _ConT
       remitter (ConT x) = Just x
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _PromotedT :: Prism' Type Name
 _PromotedT
   = prism' reviewer remitter
@@ -2610,7 +2615,6 @@ _PromotedT
       reviewer = PromotedT
       remitter (PromotedT x) = Just x
       remitter _ = Nothing
-#endif
 
 _TupleT :: Prism' Type Int
 _TupleT
@@ -2664,7 +2668,6 @@ _ListT
       remitter ListT = Just ()
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _PromotedTupleT :: Prism' Type Int
 _PromotedTupleT
   = prism' reviewer remitter
@@ -2712,7 +2715,6 @@ _LitT
       reviewer = LitT
       remitter (LitT x) = Just x
       remitter _ = Nothing
-#endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
 _InfixT :: Prism' Type (Type, Name, Type)
@@ -2766,6 +2768,58 @@ _ImplicitParamT
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,16,0)
+_ForallVisT :: Prism' Type ([TyVarBndrUnit], Type)
+_ForallVisT
+  = prism' reviewer remitter
+  where
+      reviewer (x, y) = ForallVisT x y
+      remitter (ForallVisT x y) = Just (x, y)
+      remitter _ = Nothing
+#endif
+
+#if MIN_VERSION_template_haskell(2,17,0)
+_MulArrowT :: Prism' Type ()
+_MulArrowT
+  = prism' reviewer remitter
+  where
+      reviewer () = MulArrowT
+      remitter MulArrowT = Just ()
+      remitter _ = Nothing
+#endif
+
+#if MIN_VERSION_template_haskell(2,17,0)
+_SpecifiedSpec :: Prism' Specificity ()
+_SpecifiedSpec
+  = prism' reviewer remitter
+  where
+      reviewer () = SpecifiedSpec
+      remitter SpecifiedSpec = Just ()
+      remitter _ = Nothing
+
+_InferredSpec :: Prism' Specificity ()
+_InferredSpec
+  = prism' reviewer remitter
+  where
+      reviewer () = InferredSpec
+      remitter InferredSpec = Just ()
+      remitter _ = Nothing
+#endif
+
+-- |
+-- @
+-- _PlainTV :: 'Prism'' ('TyVarBndr' flag) ('Name', flag) -- template-haskell-2.17+
+-- _PlainTV :: 'Prism''  'TyVarBndr'        'Name'        -- Earlier versions
+-- @
+#if MIN_VERSION_template_haskell(2,17,0)
+_PlainTV :: Prism' (TyVarBndr flag) (Name, flag)
+_PlainTV
+  = prism' reviewer remitter
+  where
+      reviewer (x, y) = PlainTV x y
+      remitter (PlainTV x y) = Just (x, y)
+      remitter _ = Nothing
+#else
 _PlainTV :: Prism' TyVarBndr Name
 _PlainTV
   = prism' reviewer remitter
@@ -2773,7 +2827,22 @@ _PlainTV
       reviewer = PlainTV
       remitter (PlainTV x) = Just x
       remitter _ = Nothing
+#endif
 
+-- |
+-- @
+-- _KindedTV :: 'Prism'' ('TyVarBndr' flag) ('Name', flag, 'Kind') -- template-haskell-2.17+
+-- _KindedTV :: 'Prism''  'TyVarBndr'       ('Name',       'Kind') -- Earlier versions
+-- @
+#if MIN_VERSION_template_haskell(2,17,0)
+_KindedTV :: Prism' (TyVarBndr flag) (Name, flag, Kind)
+_KindedTV
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = KindedTV x y z
+      remitter (KindedTV x y z) = Just (x, y, z)
+      remitter _ = Nothing
+#else
 _KindedTV :: Prism' TyVarBndr (Name, Kind)
 _KindedTV
   = prism' reviewer remitter
@@ -2781,6 +2850,7 @@ _KindedTV
       reviewer (x, y) = KindedTV x y
       remitter (KindedTV x y) = Just (x, y)
       remitter _ = Nothing
+#endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
 _NoSig :: Prism' FamilyResultSig ()
@@ -2799,7 +2869,7 @@ _KindSig
       remitter (KindSig x) = Just x
       remitter _ = Nothing
 
-_TyVarSig :: Prism' FamilyResultSig TyVarBndr
+_TyVarSig :: Prism' FamilyResultSig TyVarBndrUnit
 _TyVarSig
   = prism' reviewer remitter
   where
@@ -2808,7 +2878,6 @@ _TyVarSig
       remitter _ = Nothing
 #endif
 
-#if MIN_VERSION_template_haskell(2,8,0)
 _NumTyLit :: Prism' TyLit Integer
 _NumTyLit
   = prism' reviewer remitter
@@ -2824,7 +2893,6 @@ _StrTyLit
       reviewer = StrTyLit
       remitter (StrTyLit x) = Just x
       remitter _ = Nothing
-#endif
 
 #if !MIN_VERSION_template_haskell(2,10,0)
 _ClassP :: Prism' Pred (Name, [Type])
@@ -2844,7 +2912,6 @@ _EqualP
       remitter _ = Nothing
 #endif
 
-#if MIN_VERSION_template_haskell(2,9,0)
 _NominalR :: Prism' Role ()
 _NominalR
   = prism' reviewer remitter
@@ -2876,7 +2943,6 @@ _InferR
       reviewer () = InferR
       remitter InferR = Just ()
       remitter _ = Nothing
-#endif
 
 #if MIN_VERSION_template_haskell(2,12,0)
 _StockStrategy :: Prism' DerivStrategy ()
