@@ -1,9 +1,3 @@
-{-# LANGUAGE CPP #-}
-
-#ifndef MIN_VERSION_base
-#define MIN_VERSION_base(x,y,z) 1
-#endif
-
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.FilePath.Lens
@@ -23,9 +17,7 @@ module System.FilePath.Lens
   , basename, directory, extension, filename
   ) where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-#endif
+import Prelude ()
 
 import Control.Monad.State as State
 import System.FilePath
@@ -34,10 +26,14 @@ import System.FilePath
   , takeExtension, takeFileName
   )
 
+import Control.Lens.Internal.Prelude
 import Control.Lens hiding ((<.>))
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
+-- >>> import Control.Lens
+-- >>> import Control.Monad.State
+-- >>> import System.FilePath ((</>))
 
 {- NB: Be very careful if you are planning to modify the doctest output in
 this module! Path separators are OS-dependent (\\ with Windows, / with Posix),
